Scriptname _MWA_CombatArmorEquip extends ReferenceAlias  

Bool WasInterrupted = false
Bool IsInProc = false
Float Timer

Formlist Property _MWA_CombatArmorEquipFormlist Auto
Formlist Property _MWA_ImmuneList Auto

Actor Property PlayerRef Auto

Keyword Property MagicDamageFire Auto
Keyword Property MagicDamageFrost Auto
Keyword Property MagicDamageShock Auto

Sound Property _MWA_ITMGenericArmorUp Auto

_MWA_Utility Property Util Auto
_MWA_Mcm Property Menu Auto

; IN COMBAT =======================================================================

State InCombat
	Event OnBeginState()
		;Debug.Notification("In combat state")
		_MWA_CombatArmorEquipFormlist.Revert()
		RegisterForSingleUpdate(2.0)
	EndEvent

	Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
		If akBaseObject as Armor && akReference
			If !Util.HasNoStripKeyword(akBaseObject) && !Util.HasZadInvDeviceKeyword(akBaseObject) && !_MWA_ImmuneList.HasForm(akBaseObject) && !Math.LogicalAnd((akBaseObject as Armor).GetSlotMask(), 512) == 512 ; Not a shield
				If Util.IsSuitable(akReference, akBaseObject)
					_MWA_CombatArmorEquipFormlist.AddForm(akBaseObject)
					If Menu.CombatEquipWaitPerItemT
						Timer = Menu.CombatEquipTime * _MWA_CombatArmorEquipFormlist.GetSize()
					Else
						Timer = Menu.CombatEquipTime
					EndIf
					If !IsInProc
						IsInProc = true
						WasInterrupted = false
						RegForControls()
						;Debug.SendAnimationEvent(PlayerRef, "Arrok_Undress_G1") ; Better but SendAnimationEvent(IdleStop) doesn't interrupt it for some reason
						If PlayerRef.IsWeaponDrawn()
							PlayerRef.SheatheWeapon()
							Float Countdown = 2.0
							While Countdown > 0.0 && !WasInterrupted
								Utility.Wait(0.1)
								Countdown -= 0.1
							EndWhile
						EndIf
						If !WasInterrupted
							Debug.SendAnimationEvent(PlayerRef, "IdleWarmHandsCrouched")
						EndIf
						While Timer > 0.0 && !WasInterrupted
							Timer -= 0.1
							Utility.Wait(0.1)
						EndWhile
						UnRegisterForAllControls()
						Debug.SendAnimationEvent(PlayerRef, "IdleStop")
						If !WasInterrupted
							Int FinishSound = _MWA_ITMGenericArmorUp.Play(PlayerRef)
							Sound.SetInstanceVolume(FinishSound, 2.0)
						EndIf
						IsInProc = false
					EndIf
				EndIf
			EndIf
		EndIf
	EndEvent
	
	Event OnHit(ObjectReference akAggressor, Form akSource, Projectile akProjectile, bool abPowerAttack, bool abSneakAttack, bool abBashAttack, bool abHitBlocked)
		If IsInProc
			Interrupt()
		EndIf
	EndEvent
	
	Event OnMagicEffectApply(ObjectReference akCaster, MagicEffect akEffect)
		If IsInProc
			If akEffect.HasKeyword(MagicDamageFire) || akEffect.HasKeyword(MagicDamageFrost) || akEffect.HasKeyword(MagicDamageShock)
				Interrupt()
			EndIf
		EndIf
	EndEvent
	
	Event OnControlDown(string control)
		If IsInProc
			Interrupt()
		EndIf
	EndEvent
	
	Event OnUpdate()
		If PlayerRef.IsInCombat()
			RegisterForSingleUpdate(2.0)
		Else
			UnRegisterForUpdate()
			GoToState("")
		EndIf
	EndEvent
EndState

Function Interrupt()
	WasInterrupted = true
	UnRegisterForAllControls()
	Debug.SendAnimationEvent(PlayerRef, "IdleStop")
	CancelEquips()
EndFunction

; NOT IN COMBAT =====================================================================

Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
EndEvent

Event OnHit(ObjectReference akAggressor, Form akSource, Projectile akProjectile, bool abPowerAttack, bool abSneakAttack, bool abBashAttack, bool abHitBlocked)
EndEvent

Event OnMagicEffectApply(ObjectReference akCaster, MagicEffect akEffect)
EndEvent

Event OnControlDown(string control)
EndEvent

Event OnBeginState()
	;Debug.Notification("NOT in combat state")
EndEvent

; FUNCTIONS ===========================================================================

Function CancelEquips()
	Int i = _MWA_CombatArmorEquipFormlist.GetSize()
	While i > 0
		i -= 1
		PlayerRef.UnequipItem(_MWA_CombatArmorEquipFormlist.GetAt(i), abPreventEquip = false, abSilent = true)
	EndWhile
	_MWA_CombatArmorEquipFormlist.Revert()
EndFunction

Function RegForControls()
	RegisterForControl("Move")
	RegisterForControl("Forward")
	RegisterForControl("Back")
	RegisterForControl("Strafe Left")
	RegisterForControl("Strafe Right")
EndFunction
