Scriptname _MWA_DurRepairSpecific extends ObjectReference  

; Script to repair the durability of specific items given to tailors/blacksmiths

_MWA_Utility Property Util Auto
_MWA_Init Property Init Auto

GlobalVariable Property _MWA_FixCost Auto

Actor Property PlayerRef Auto

MiscObject Property Gold001 Auto

Message Property _MWA_RepairConfirm Auto

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	If akItemReference != None && StringUtil.Find(akItemReference, "in container") == -1 ; Good ObjRef
		If Util.IsCorrectCraftsman(akItemReference)
			Int RepairCost = Util.CostDurRepairSpecific(akBaseItem, akItemReference)
			If RepairCost > 0
				Debug.Notification(akBaseItem.GetName() + " will cost " + RepairCost + " gold to be repaired")
				StorageUtil.FormListAdd(none, "_MWA_ForRepairList",  akItemReference, allowDuplicate = false)
				_MWA_FixCost.SetValueInt(_MWA_FixCost.GetValueInt() + RepairCost)
			Else
				Debug.Notification(akBaseItem.GetName() + " doesn't need to be repaired")
				GiveBack(akBaseItem, akItemReference, aiItemCount)
			EndIf
		Else
			GiveBack(akBaseItem, akItemReference, aiItemCount)
			Debug.Notification("This is not the right craftman for " + akBaseItem.GetName())
		EndIf
	Else
		Debug.Trace("_MWA_: Error: _MWA_DurRepairSpecific received a unusable ObjRef for: " + akBaseItem.GetName())
	EndIf
EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)

EndEvent

State IgnoreRemoval
	Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	
	EndEvent
EndState

Function GiveBack(Form akBaseItem, ObjectReference akItemReference, Int aiItemCount)
	GoToState("IgnoreRemoval")
	If akItemReference
		Self.RemoveItem(akItemReference, aiItemCount, true, PlayerRef)
	Else
		Self.RemoveItem(akBaseItem, aiItemCount, true, PlayerRef)
	EndIf
	GoToState("")
EndFunction

Event OnActivate(ObjectReference akActionRef)
	If akActionRef == PlayerRef
		RegisterForMenu("ContainerMenu")
	EndIf
EndEvent

Event OnMenuClose(String MenuName)
	If Self.GetNumItems() > 0
		If PlayerRef.GetItemCount(Gold001) >= _MWA_FixCost.GetValueInt()
		
			Int Button = _MWA_RepairConfirm.Show(afArg1 = _MWA_FixCost.GetValue())
			If Button == 0 ; Proceed
				PlayerRef.RemoveItem(akItemToRemove = Gold001, aiCount = _MWA_FixCost.GetValueInt(), abSilent = false, akOtherContainer = Init.RepairActor)
				DoRepairs()
			Else ; cancel
				GoToState("IgnoreRemoval")
				Self.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
				GoToState("")
			Endif
		Else
			Debug.Notification("You can't afford that")
			GoToState("IgnoreRemoval")
			Self.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
			GoToState("")
		EndIf
		
		; Reset
		StorageUtil.FormListClear(none, "_MWA_ForRepairList")
		GoToState("IgnoreRemoval")
		Self.RemoveAllItems()
		GoToState("")
		_MWA_FixCost.SetValueInt(0)
	EndIf
	UnregisterForMenu("ContainerMenu")
EndEvent

Function DoRepairs()
	Int i = StorageUtil.FormListCount(none, "_MWA_ForRepairList") - 1
	ObjectReference akReference
	While i >= 0
		akReference = StorageUtil.FormListGet(none, "_MWA_ForRepairList", i) as ObjectReference
		StorageUtil.SetIntValue(akReference, "_MWA_Durability", (StorageUtil.GetIntValue(akReference, "_MWA_DurabilityMax")))
		Util.DispItemDurability(akReference, akReference.GetBaseObject(), 0.0)
		i -= 1
	EndWhile
	Self.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
	StorageUtil.FormListClear(none, "_MWA_ForRepairList")
	Util.ResumeInventoryTracking()
EndFunction
