Scriptname _MWA_FitContainer extends ObjectReference  

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	If (!akBaseItem as Armor) || akBaseItem.HasKeyword(ArmorShield) || akBaseItem.HasKeyword(_MWA_Broken)
		Debug.Notification(akBaseItem.GetName() + " is not adjustable")
		GiveBack(akBaseItem, akItemReference, aiItemCount)
		
	Else
		Int ItemRepairCost
		Int ArmorClass = (akBaseItem as Armor).GetWeightClass()
		If (Init.RepairActor.IsInFaction(_MWA_TailorFaction) && (ArmorClass == 0 || ArmorClass == 1))
			Debug.Notification("Only a smith can adjust this item")
			GiveBack(akBaseItem, akItemReference, aiItemCount)
		ElseIf Init.RepairActor.IsInFaction(JobBlacksmithFaction) && ArmorClass == 2
			Debug.Notification("Only a tailor can adjust this item")
			GiveBack(akBaseItem, akItemReference, aiItemCount)
		Else
		
			Float Fit = StorageUtil.GetFloatValue(akItemReference, "_MWA_ArmorFit", 101)
			Bool IsGenderMatch = Util.IsGenderMatch(akItemReference, Fit, true)
			Bool IsWeightMatch = Util.IsWeightMatch(akBaseItem, akItemReference, Fit, true)
			RegisterForMenu("ContainerMenu")
			
			If IsGenderMatch && IsWeightMatch
				Debug.Notification(akBaseItem.GetName() + " doesn't need adjustment")
				GiveBack(akBaseItem, akItemReference, aiItemCount)
			
			Else ; Needs adjustment
			
				; Get refit costs
				Int BaseCost = akBaseItem.GetGoldValue()
				
				Int EnchantCost = GetEnchantCost(akItemReference, akBaseItem)
				
				
				If !IsWeightMatch ; Weight adjustment
					ItemRepairCost = Menu.WeightAdjustCostMin + Math.Ceiling((BaseCost * Menu.WeightAdjustCostBase) + (EnchantCost * Menu.WeightAdjustCostEnchant))
				
				Else ; Gender adjustment
					ItemRepairCost = Menu.GenderAdjustCostMin + Math.Ceiling((BaseCost * Menu.GenderAdjustCostBase) + (EnchantCost * Menu.GenderAdjustCostEnchant))
				EndIf
			EndIf

			StorageUtil.SetIntValue(akItemReference, "_MWA_RefitCost", ItemRepairCost)
			_MWA_FixCost.SetValueInt(_MWA_FixCost.GetValueInt() + ItemRepairCost)
			StorageUtil.FormListAdd(none, "_MWA_ForRepairList",  akItemReference, allowDuplicate = false)
			Debug.Notification("Adjust cost: " + ItemRepairCost + ". Total cost: " + _MWA_FixCost.GetValueInt() + ". Your gold: " + PlayerRef.GetItemCount(Gold001) + ". Remaining gold: " + (PlayerRef.GetItemCount(Gold001) - _MWA_FixCost.GetValueInt()))
		EndIf
	EndIf		
EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	If (!akBaseItem as Armor) || akBaseItem.HasKeyword(ArmorShield) || akBaseItem.HasKeyword(_MWA_Broken)
		; Do nothing
		
	Else
		Int ItemRepairCost = StorageUtil.GetIntValue(akItemReference, "_MWA_RefitCost", ItemRepairCost)
		_MWA_FixCost.SetValueInt(_MWA_FixCost.GetValueInt() - ItemRepairCost)
		StorageUtil.FormListRemove(none, "_MWA_ForRepairList", akItemReference, allInstances = true)
		StorageUtil.UnSetIntValue(akItemReference, "_MWA_RefitCost")
		Debug.Notification("Refit cost reduced by: " + ItemRepairCost + ". Total cost: " + _MWA_FixCost.GetValueInt() + ". Your gold: " + PlayerRef.GetItemCount(Gold001) + ". Remaining gold: " + (PlayerRef.GetItemCount(Gold001) - _MWA_FixCost.GetValueInt()))
	EndIf
EndEvent

State IgnoreRemoval
	Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	
	EndEvent
EndState

Event OnMenuClose(String MenuName)
	If _MWA_EnchantTestSubjectRef.GetNumItems() > 0
		If PlayerRef.GetItemCount(Gold001) >= _MWA_FixCost.GetValueInt()
			Int Button = _MWA_RepairConfirm.Show(afArg1 = _MWA_FixCost.GetValue())
			If Button == 0 ; Proceed
				PlayerRef.RemoveItem(akItemToRemove = Gold001, aiCount = _MWA_FixCost.GetValueInt(), abSilent = false, akOtherContainer = Init.RepairActor)
				DoRefit()
			Else ; cancel
				GoToState("IgnoreRemoval")
				_MWA_EnchantTestSubjectRef.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
				GoToState("")
			Endif
		Else
			Debug.Notification("You can't afford that")
			GoToState("IgnoreRemoval")
			_MWA_EnchantTestSubjectRef.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
			GoToState("")
		EndIf
		
		; Reset
		StorageUtil.FormListClear(none, "_MWA_ForRepairList")
		GoToState("IgnoreRemoval")
		_MWA_EnchantTestSubjectRef.RemoveAllItems()
		GoToState("")
		_MWA_FixCost.SetValueInt(0)
	EndIf
	UnregisterForMenu("ContainerMenu")
EndEvent

Int Function GetEnchantCost(ObjectReference akItemReference, Form akBaseItem)
	Int Result = 0
	_MWA_EnchantTestSubjectRef.MoveTo(PlayerRef, afZOffset = -200.0)
	_MWA_EnchantTestSubjectRef.EquipItem(akBaseItem)
	Enchantment E = Util.GetItemEnchantment(akItemReference, 0, _MWA_EnchantTestSubjectRef)
	If E != None
		Int HandSlot = 0
		Int SlotMask = (akBaseItem as Armor).GetSlotMask()

		Float ChargeMax = WornObject.GetItemMaxCharge(_MWA_EnchantTestSubjectRef, HandSlot, SlotMask) ; Don't think armors can have charge but whatever
		;Debug.MessageBox(E + " - " + ChargeMax)
		Result = Util.GetEnchantedGoldValue(E, akBaseItem, akItemReference, ChargeMax)
	EndIf
	_MWA_EnchantTestSubjectRef.MoveTo(_MWA_EnchantTestSubjectHome)
	Return Result
EndFunction

Function GiveBack(Form akBaseItem, ObjectReference akItemReference, Int aiItemCount)
	GoToState("IgnoreRemoval")
	If akItemReference
		Self.RemoveItem(akItemReference, aiItemCount, true, PlayerRef)
	Else
		Self.RemoveItem(akBaseItem, aiItemCount, true, PlayerRef)
	EndIf
	GoToState("")
EndFunction

Function DoRefit()
	ObjectReference ObjRef
	Int Gender = PlayerRef.GetActorBase().GetSex()
	If Gender == 0
		Gender = -1
	EndIf
	Float PlayerFit = Gender * PlayerRef.GetActorBase().GetWeight()
	Int i = StorageUtil.FormlistCount(none, "_MWA_ForRepairList")
	While i > 0
		i -= 1
		ObjRef = StorageUtil.FormlistGet(none, "_MWA_ForRepairList", i) as ObjectReference
		StorageUtil.SetIntValue(ObjRef, "_MWA_Durability", (StorageUtil.GetIntValue(ObjRef, "_MWA_DurabilityMax")))
		StorageUtil.SetFloatValue(ObjRef, "_MWA_ArmorFit", (PlayerFit + 0.1))
		;Debug.MessageBox(ObjRef + " - fit: " + PlayerFit)
	EndWhile
	_MWA_EnchantTestSubjectRef.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
EndFunction

Faction Property _MWA_TailorFaction Auto
Faction Property JobBlacksmithFaction Auto

Keyword Property ArmorShield Auto
Keyword Property _MWA_Broken Auto

GlobalVariable Property _MWA_FixCost Auto

MiscObject Property Gold001 Auto

ObjectReference Property _MWA_EnchantTestSubjectHome Auto

Actor Property PlayerRef Auto
Actor Property _MWA_EnchantTestSubjectRef Auto

Message Property _MWA_RepairConfirm Auto

_MWA_Utility Property Util Auto
_MWA_Init Property Init Auto
_MWA_Mcm Property Menu Auto
