Scriptname _MWA_Init extends Quest  Conditional

; EVENTS =======================================================================================================

Event OnInit()
	; HearthFires
	If Game.GetModByName("HearthFires.esm") != 255 ; Adds wooden sword to junk list
		Util._MWA_JunkObjects.AddForm(Game.GetFormFromFile(0x00004D91, "HearthFires.esm"))
	EndIf
	
	SoftDependencyCheck()
EndEvent

; FUNCTIONS ====================================================================================================

Function PlayerLoadsGame()
	SoftDependencyCheck()
	SexlabInterface.PlayerLoadsGame()
	Devious.PlayerLoadsGame()
EndFunction

Function SoftDependencyCheck()
	Debug.Trace("_MWA_: Soft dependency check Begin ===========================")
	
	; Dawnguard
	If Game.GetModByName("Dawnguard.esm") != 255
		IsDawnguardInstalled = true
		Util.DLC1ArmorMaterialDawnguard = Game.GetFormFromFile(0x00012CCD, "Dawnguard.esm") as Keyword
		Util.DLC1ArmorMaterielFalmerHeavyOriginal = Game.GetFormFromFile(0x00012CD0, "Dawnguard.esm") as Keyword ; Why is falmer armor such a mess
		Util.DLC1ArmorMaterielFalmerHeavy = Game.GetFormFromFile(0x00012CCF, "Dawnguard.esm") as Keyword
		Util.DLC1ArmorMaterialFalmerHardened = Game.GetFormFromFile(0x00012CCE, "Dawnguard.esm") as Keyword
		Util.DLC1ArmorMaterialVampire = Game.GetFormFromFile(0x0001463E, "Dawnguard.esm") as Keyword
		Util.DLC1WeapMaterialDragonbone = Game.GetFormFromFile(0x00019822, "Dawnguard.esm") as Keyword
		Util.DLC1DawnguardItem = Game.GetFormFromFile(0x0000D839, "Dawnguard.esm") as Keyword
	Else
		IsDawnguardInstalled = false
	EndIf
	Debug.Trace("_MWA_: Dawnguard Installed: " + IsDawnguardInstalled)
	
	; Dragonborn
	If Game.GetModByName("Dragonborn.esm") != 255
		IsDragonbornInstalled = true
		Util.DLC2ArmorMaterialBonemoldHeavy = Game.GetFormFromFile(0x00024101, "Dragonborn.esm") as Keyword
		Util.DLC2ArmorMaterialChitinLight = Game.GetFormFromFile(0x00024102, "Dragonborn.esm") as Keyword
		Util.DLC2ArmorMaterialChitinHeavy = Game.GetFormFromFile(0x00024103, "Dragonborn.esm") as Keyword
		Util.DLC2ArmorMaterialNordicHeavy = Game.GetFormFromFile(0x00024105, "Dragonborn.esm") as Keyword
		Util.DLC2ArmorMaterialStalhrimLight = Game.GetFormFromFile(0x00024107, "Dragonborn.esm") as Keyword
		Util.DLC2WeaponMaterialNordic = Game.GetFormFromFile(0x00026230, "Dragonborn.esm") as Keyword
		Util.DLC2WeaponMaterialStalhrim = Game.GetFormFromFile(0x0002622F, "Dragonborn.esm") as Keyword
	Else
		IsDragonbornInstalled = false
	EndIf
	Debug.Trace("_MWA_: Dragonborn Installed: " + IsDragonbornInstalled)
	
	; Sexlab
	If Game.GetModByName("SexLab.esm") != 255
		IsSexlabInstalled = true
		SexlabNoStripKw = Game.GetFormFromFile(0x2F16E, "Sexlab.esm") as Keyword
		;SexLabAnimatingFaction = Game.GetFormFromFile(0xE50F, "Sexlab.esm") as Faction
	Else
		IsSexlabInstalled = false
	EndIf
	Debug.Trace("_MWA_: Sexlab Installed: " + IsSexlabInstalled)
	
	; Devious Devices
	If Game.GetModByName("Devious Devices - Integration.esm") != 255
		IsDdInstalled = true
		zad_InventoryDevice = Game.GetFormFromFile(0x02B5F0, "Devious Devices - Integration.esm") as Keyword
	Else
		IsDdInstalled = false
	EndIf
	Debug.Trace("_MWA_: Devious Devices Installed: " + IsDdInstalled)
	
	; Requiem
	If Game.GetModByName("Requiem.esp") != 255
		IsRequiemInstalled = true
		If !RequiemInitialized
			InitializeRequiem()
		EndIf
	Else
		IsRequiemInstalled = false
		If RequiemInitialized
			UnInstallRequiem()
		EndIf
	EndIf
	Debug.Trace("_MWA_: Requiem Installed: " + IsRequiemInstalled)
	
	Debug.Trace("_MWA_: Soft dependency check End ============================")
EndFunction

Float Function GetNextTemperDegrade(Float ObjectHealth)
	If ObjectHealth >= TemperFloatLegendary
		Return TemperFloatLegendary
	ElseIf ObjectHealth >= TemperFloatEpic
		Return TemperFloatEpic
	ElseIf ObjectHealth >= TemperFloatFlawless
		Return TemperFloatFlawless
	ElseIf ObjectHealth >= TemperFloatExquisite
		Return TemperFloatExquisite
	ElseIf ObjectHealth >= TemperFloatSuperior
		Return TemperFloatSuperior
	ElseIf ObjectHealth >= TemperFloatFine
		Return TemperFloatFine
	Else
		Return Util.TemperFloor
	EndIf
EndFunction

Float Function GetBumpedTemperValue(Float ObjectHealth) ; Returns the Object health just before the next temper level to give a full range of temper degradation
	If ObjectHealth >= TemperFloatLegendary
		Return TemperFloatLegendary + Menu.ItemHealthPerTier - 0.001
	ElseIf ObjectHealth >= TemperFloatEpic
		Return TemperFloatEpic + Menu.ItemHealthPerTier - 0.001
	ElseIf ObjectHealth >= TemperFloatFlawless
		Return TemperFloatFlawless + Menu.ItemHealthPerTier - 0.001
	ElseIf ObjectHealth >= TemperFloatExquisite
		Return TemperFloatExquisite + Menu.ItemHealthPerTier - 0.001
	ElseIf ObjectHealth >= TemperFloatSuperior
		Return TemperFloatSuperior + Menu.ItemHealthPerTier - 0.001
	ElseIf ObjectHealth >= TemperFloatFine
		Return TemperFloatFine + Menu.ItemHealthPerTier - 0.001
	Else
		Return 1.0
	EndIf
EndFunction

Float Function GetUpgradedTemperHealth(Float RecordedHealth) ; Returns the next temper level float at which the item should be considered as having been upgraded
	If RecordedHealth >= TemperFloatLegendary
		Return 1000.0
		
	ElseIf RecordedHealth >= TemperFloatEpic
		Return TemperFloatLegendary
		
	ElseIf RecordedHealth >= TemperFloatFlawless
		Return TemperFloatEpic
		
	ElseIf RecordedHealth >= TemperFloatExquisite
		Return TemperFloatFlawless
		
	ElseIf RecordedHealth >= TemperFloatSuperior
		Return TemperFloatExquisite
		
	ElseIf RecordedHealth >= TemperFloatFine
		Return TemperFloatSuperior
		
	Else
		Return TemperFloatFine
	EndIf	
EndFunction

Function InitializeRequiem()
	Debug.Notification("_MWA_: Changing degradation settings to fit Requiem")
	Menu.ItemHealthPerTier = 1.2
	;CombatArmor.TemperFloor = 1.0
	;CombatWeapon.TemperFloor = 1.0
	;Util.TemperFloor = 1.099999
	Util.TemperFloor = 1.1
	TemperFloatFine = 1.1
	TemperFloatSuperior = 2.2
	TemperFloatExquisite = 3.4
	TemperFloatFlawless = 4.6
	TemperFloatEpic = 5.8
	TemperFloatLegendary = 7.0
	
	TemperStringFine = "Well-Made"
	TemperStringSuperior = "High-Grade"
	TemperStringExquisite = "First-Rate"
	TemperStringFlawless = "Exquisite"
	TemperStringEpic = "Master Work"
	TemperStringLegendary = "Legendary"
	RequiemInitialized = true
	
	Menu.TemperDegrade = 0.03
	Menu.TemperDegradePowerA = 0.06
	Menu.WeaponDegrade = 0.006
	Menu.WeaponDegradePower = 0.012
	Menu.TimeDegradeDamage = 0.06
EndFunction

Function UnInstallRequiem()
	Menu.ItemHealthPerTier = 0.1
	;CombatArmor.TemperFloor = 1.0
	;CombatWeapon.TemperFloor = 1.0
	;Util.TemperFloor = 1.099999
	Util.TemperFloor = 1.1
	TemperFloatFine = 1.1
	TemperFloatSuperior = 1.2
	TemperFloatExquisite = 1.3
	TemperFloatFlawless = 1.4
	TemperFloatEpic = 1.5
	TemperFloatLegendary = 1.6
	
	TemperStringFine = "Fine"
	TemperStringSuperior = "Superior"
	TemperStringExquisite = "Exquisite"
	TemperStringFlawless = "Flawless"
	TemperStringEpic = "Epic"
	TemperStringLegendary = "Legendary"
	RequiemInitialized = false
	
	Menu.TemperDegrade = 0.005
	Menu.TemperDegradePowerA = 0.010
	Menu.WeaponDegrade = 0.001
	Menu.WeaponDegradePower = 0.002
	Menu.TimeDegradeDamage = 0.0050
EndFunction

; SCRIPT VARIABLES =============================================================================================
Bool RequiemInitialized = false

; PROPERTIES ===================================================================================================

Weapon Property OldLeftWeapon Auto Hidden
Weapon Property OldRightWeapon Auto Hidden

Int Property BrokenTrackAliasCount Auto Hidden
Int Property NpcLootAliasIndex = 0 Auto Hidden
Int Property NpcLootAliasCount Auto Hidden

Bool Property IsDawnguardInstalled = false Auto Hidden
Bool Property IsDragonbornInstalled = false Auto Hidden
Bool Property IsSexlabInstalled = false Auto Hidden
Bool Property IsDdInstalled = false Auto Hidden Conditional
Bool Property IsRequiemInstalled = false Auto Hidden

Float Property TemperFloatFine = 1.1 Auto Hidden
Float Property TemperFloatSuperior = 1.2 Auto Hidden
Float Property TemperFloatExquisite = 1.3 Auto Hidden
Float Property TemperFloatFlawless = 1.4 Auto Hidden
Float Property TemperFloatEpic = 1.5 Auto Hidden
Float Property TemperFloatLegendary = 1.6 Auto Hidden

String Property TemperStringFine = "Fine" Auto Hidden
String Property TemperStringSuperior = "Superior" Auto Hidden
String Property TemperStringExquisite = "Exquisite" Auto Hidden
String Property TemperStringFlawless = "Flawless" Auto Hidden
String Property TemperStringEpic = "Epic" Auto Hidden
String Property TemperStringLegendary = "Legendary" Auto Hidden

Keyword Property SexlabNoStripKw Auto Hidden
Keyword Property zad_InventoryDevice Auto Hidden

;Faction Property SexLabAnimatingFaction Auto Hidden

Actor Property RepairActor Auto Hidden
Actor Property PlayerRef Auto

Spell Property _MWA_CellChangeDetectSpell Auto

_MWA_Utility Property Util Auto
_MWA_CombatArmor Property CombatArmor Auto
_MWA_CombatWeapon Property CombatWeapon Auto
_MWA_Mcm Property Menu Auto

_MWA_InterfaceSexlab Property SexlabInterface Auto
_MWA_InterfaceDeviousDevices Property Devious Auto
