Scriptname _MWA_Main extends ReferenceAlias

Float Version

Event OnInit()
	Version = 0.43
	_MWA_CellChangeDetectObjRef.MoveTo(PlayerRef)
	PlayerRef.AddSpell(_MWA_CellChangeDetectSpell, false)
	PlayerRef.AddSpell(_MWA_IsPlayerInCombat, false)
	PlayerRef.AddSpell(_MWA_DetectEquipmentSpell)
	RecheckAliasCount()
	ImportImmuneList()
	AddCommonImmunes()
EndEvent

Event OnPlayerLoadGame()
	;Debug.Trace("_MWA_: LOAD GAME ================================================================================")
	_MWA_CellChangeDetectObjRef.MoveTo(PlayerRef)
	Init.PlayerLoadsGame()
	Util.PlayerLoadsGame()
	SanityChecker.PlayerLoadsGame()
	Api.PlayerLoadsGame()
	InvUtil.PlayerLoadsGame()
	Menu.PlayerLoadsGame()
	If Game.GetModByName("SL Survival.esp") != 255
		_MWA_SurvivalFittedArmorTrackQuest.Start()
	
	Else
		_MWA_SurvivalFittedArmorTrackQuest.Stop()
	EndIf
	DoUpgrade()
EndEvent

Function DoUpgrade()
	If Version < 0.43		
		If Version < 0.40
			Menu.SetupMenuArrays()
			AddCommonImmunes()
			Debug.Notification("MWA updated to 0.40")
			Version = 0.40
		EndIf
		If Version < 0.41
			If Menu.RemoveFoodCANs > 0.0
				PlayerRef.AddPerk(Menu._MWA_FoodRemovalPerk)
			EndIf
			Debug.Notification("MWA updated to 0.41")
			Version = 0.41
		EndIf
		If Version < 0.42
			SanityChecker.SetupSlotMasks()
			_MWA_DialogueQuest.Stop()
			Utility.Wait(0.1)
			_MWA_DialogueQuest.Start()
			_MWA_DialogueQuest.UpdateCurrentInstanceGlobal(Menu._MWA_SanctCost)
			Menu.Devious.PlayerLoadsGame()
			Version = 0.42
			Debug.Notification("MWA updated to 0.42")
		EndIf
		If Version < 0.43
			Menu.ToggleLockpickRemoval()
			Debug.Notification("MWA updated to 0.43")
			Version = 0.43
		EndIf
	EndIf
EndFunction

Function ImportImmuneList()
	Int i = JsonUtil.FormlistCount("MWA/ImmuneObjects.json", "ImmuneList")
	Form akBaseObject
	While i > 0 
		i -= 1
		akBaseObject = JsonUtil.FormlistGet("MWA/ImmuneObjects.json", "ImmuneList", i)
		If akBaseObject != None
			_MWA_ImmuneList.AddForm(akBaseObject)
		EndIf
	EndWhile
EndFunction

Function AddCommonImmunes()
	; Frostfall
	If Game.GetModByName("Frostfall.esp") != 255
		_MWA_ImmuneList.AddForm(Game.GetFormFromFile(0x05711D, "Frostfall.esp")) ; Bound Cloak, Lesser
		_MWA_ImmuneList.AddForm(Game.GetFormFromFile(0x057BE8, "Frostfall.esp")) ; Bound Cloak, Greater
	EndIf
EndFunction
 
Function RecheckAliasCount() ; Check if the number of aliases has increased
	Init.BrokenTrackAliasCount = _MWA_BrokenTrack.GetNumAliases()
	Init.NpcLootAliasCount = _MWA_NpcLootTrack.GetNumAliases()
EndFunction

Function ShutdownMod()
	Debug.Notification("MWA shutting down")
	PlayerRef.RemoveSpell(_MWA_CellChangeDetectSpell)
	PlayerRef.RemoveSpell(_MWA_IsPlayerInCombat)

	ResetObjects(_MWA_PlayerInventoryAliases, DeleteObjRef = false)
	ResetObjects(_MWA_BrokenTrack, DeleteObjRef = true)
	ResetObjects(_MWA_NpcLootTrack, DeleteObjRef = false)
	
	_MWA_PlayerInventoryAliases.Stop()
	_MWA_ObjectEquipMonitorQuest.Stop()
	_MWA_BrokenTrack.Stop()
	_MWA_BrokenUpdateQuest.Stop()
	_MWA_CombatArmorQuest.Stop()
	_MWA_CombatWeaponQuest.Stop()
	_MWA_DegradeOverTimeQuest.Stop()
	_MWA_DialogueQuest.Stop()
	_MWA_NpcDegradationQuest.Stop()
	_MWA_NpcLootTrack.Stop()
	_MWA_TemperingQuest.Stop()
	_MWA_SurvivalFittedArmorTrackQuest.Stop()
	
	StorageUtil.FormListClear(none, "_MWA_WornDestructibles")
	StorageUtil.FormListClear(none, "_MWA_BrokenTrackAliasList")
	
	; Shutdown search triggers
	Int i = 0
	ObjectReference ObjRef
	While i < ArmorDest.SearchBoxes.Length
		ObjRef = ArmorDest.SearchBoxes[i]
		If ObjRef != None
			(ObjRef as _MWA_ObjectSearch).ShutdownTrigger()
		EndIf
		i += 1
	EndWhile
	Debug.Notification("MWA shut down complete")
EndFunction

Function StartUpMod()
	Debug.Notification("MWA starting up")
	_MWA_CellChangeDetectObjRef.MoveTo(PlayerRef)
	PlayerRef.AddSpell(_MWA_CellChangeDetectSpell, false)
	PlayerRef.AddSpell(_MWA_IsPlayerInCombat, false)
	RecheckAliasCount()
	
	_MWA_PlayerInventoryAliases.Start()
	_MWA_ObjectEquipMonitorQuest.Start()
	_MWA_BrokenTrack.Start()
	_MWA_BrokenUpdateQuest.Start()
	_MWA_CombatArmorQuest.Start()
	_MWA_CombatWeaponQuest.Start()
	_MWA_DegradeOverTimeQuest.Start()
	_MWA_DialogueQuest.Start()
	_MWA_NpcDegradationQuest.Start()
	_MWA_NpcLootTrack.Start()
	_MWA_TemperingQuest.Start()
	Debug.Notification("MWA start up complete")
EndFunction

Function ResetObjects(Quest QuestToProc, Bool DeleteObjRef = false)
	Int AliasCount = QuestToProc.GetNumAliases()
	Int i = 0
	ObjectReference ObjRef
	While i < AliasCount
		ObjRef = (QuestToProc.GetNthAlias(i) as ReferenceAlias).GetReference()
		If ObjRef != None
			If StringUtil.Find(ObjRef, "in container") == -1
				ObjRef.SetDisplayName(ObjRef.GetBaseObject().GetName())
				
				StorageUtil.ClearAllObjPrefix(ObjRef, "_MWA_")
				;/
				StorageUtil.UnsetIntValue(ObjRef, "_MWA_Durability")
				StorageUtil.UnsetIntValue(ObjRef, "_MWA_DurabilityMax")
				StorageUtil.UnsetFloatValue(ObjRef, "_MWA_DurabilityBonus")
				StorageUtil.UnsetFormValue(ObjRef, "_MWA_OriginalArmor")
				StorageUtil.UnsetFormValue(ObjRef, "_MWA_OriginalEnchantment")
				StorageUtil.UnsetFloatValue(ObjRef, "_MWA_OriginalCharge")
				StorageUtil.UnsetFloatValue(ObjRef, "_MWA_OriginalChargeMax")
				StorageUtil.UnsetFloatValue(ObjRef, "_MWA_OriginalValue")
				StorageUtil.UnsetFloatValue(ObjRef, "_MWA_OriginalValueEnchant")
				StorageUtil.UnsetFloatValue(ObjRef, "_MWA_LastHealth")
				/;
				If DeleteObjRef
					ObjRef.Delete()
				EndIf
			Else
				Debug.Trace("_MWA_: Error: " + ObjRef + " is not a usable reference")
			EndIf
		EndIf
		i += 1
	EndWhile
EndFunction

Function ResumeDegradation()
	_MWA_CombatArmorQuest.Start()
	_MWA_CombatWeaponQuest.Start()
	_MWA_DegradeOverTimeQuest.Start()
EndFunction

Function PauseDegradation()
	_MWA_CombatArmorQuest.Stop()
	_MWA_CombatWeaponQuest.Stop()
	_MWA_DegradeOverTimeQuest.Stop()
EndFunction

Function ToggleNpcDegradeScan()
	If Menu.NpcDegradationScanT
		_MWA_CellChangeDetectObjRef.MoveTo(PlayerRef)
		PlayerRef.AddSpell(_MWA_CellChangeDetectSpell, false)
		PlayerRef.AddSpell(_MWA_IsPlayerInCombat, false)
	Else
		PlayerRef.RemoveSpell(_MWA_CellChangeDetectSpell)
		PlayerRef.RemoveSpell(_MWA_IsPlayerInCombat)
	EndIf
EndFunction

ObjectReference Property _MWA_CellChangeDetectObjRef Auto

Quest Property _MWA_BrokenTrack Auto
Quest Property _MWA_NpcLootTrack Auto
Quest Property _MWA_ObjectEquipMonitorQuest Auto
Quest Property _MWA_BrokenUpdateQuest Auto
Quest Property _MWA_CombatArmorQuest Auto
Quest Property _MWA_CombatWeaponQuest Auto
Quest Property _MWA_DegradeOverTimeQuest Auto
Quest Property _MWA_DialogueQuest Auto
Quest Property _MWA_NpcDegradationQuest Auto
Quest Property _MWA_TemperingQuest Auto
Quest Property _MWA_PlayerInventoryAliases Auto
Quest Property _MWA_SurvivalFittedArmorTrackQuest Auto

Formlist Property _MWA_ImmuneList Auto

Spell Property _MWA_CellChangeDetectSpell Auto
Spell Property _MWA_IsPlayerInCombat Auto
Spell Property _MWA_DetectEquipmentSpell Auto

Actor Property PlayerRef Auto

_MWA_Init Property Init Auto
_MWA_ArmorDestruction Property ArmorDest Auto
_MWA_Mcm Property Menu Auto
_MWA_Utility Property Util Auto
_MWA_Api Property Api Auto
_MWA_PlayerInventoryUtil Property InvUtil Auto
_MWA_SanityChecker Property SanityChecker Auto
