Scriptname _MWA_NpcDegradationAlias extends ReferenceAlias  

ObjectReference ObjRef

Event OnDying(Actor akKiller)
	DegradeCorpse()
EndEvent

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)

EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)

EndEvent


State Dead
	Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)

	EndEvent

	Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
		;Debug.Messagebox(akBaseItem + ". " + akItemReference + ". " + akDestContainer)
		ObjRef = akItemReference
	EndEvent
EndState

State Alive
	Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
		If DestructTypes.IsDestructible(akBaseItem)
			Actor Corpse = Self.GetReference() as Actor
			StorageUtil.FormListAdd(Corpse, "_MWA_DegradeList", akBaseItem, allowDuplicate = false)
			StorageUtil.SetIntValue(Corpse, "_MWA_NpcItemCount", Corpse.GetNumItems())
		EndIf
	EndEvent

	Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
		Actor Corpse = Self.GetReference() as Actor
		If Corpse.GetItemCount(akBaseItem) < 1
			If DestructTypes.IsDestructible(akBaseItem)
				
				StorageUtil.FormListRemove(Corpse, "_MWA_DegradeList", akBaseItem, allInstances = false)
				StorageUtil.SetIntValue(Corpse, "_MWA_NpcItemCount", Corpse.GetNumItems())
			EndIf
		EndIf
	EndEvent
EndState

Event OnInit()
	RemoveAllInventoryEventFilters()
	AddInventoryEventFilter(_MWA_DummyArmor)
	Actor SoonToBeCorpse = Self.GetReference() as Actor
	If SoonToBeCorpse != None
		GoToState("Alive")
		Int StoredItemCount = StorageUtil.GetIntValue(SoonToBeCorpse, "_MWA_NpcItemCount", -1)
		If StoredItemCount !=  SoonToBeCorpse.GetNumItems() ; Then ObjRef has not yet had it's inventory filtered or it's inventory has changed
			StorageUtil.FormListClear(SoonToBeCorpse, "_MWA_DegradeList")
			;Debug.Trace("_MWA_: Degrade: Init: " + SoonToBeCorpse)
			Int i = SoonToBeCorpse.GetNumItems()
			Form CurrentForm
			StorageUtil.SetIntValue(SoonToBeCorpse, "_MWA_NpcItemCount", i)
			
			If Menu.RemoveFoodCANs > 0.0 ; Do food and objects
				While i > 0
					i -= 1
					CurrentForm = SoonToBeCorpse.GetNthForm(i)
					If DestructTypes.IsDestructible(CurrentForm)
						StorageUtil.FormListAdd(SoonToBeCorpse, "_MWA_DegradeList", CurrentForm, allowDuplicate = false)
						
					ElseIf CurrentForm as Potion && (CurrentForm.HasKeyword(VendorItemFood) || CurrentForm.HasKeyword(VendorItemFoodRaw))
						If Menu.RemoveFoodCANs >= Utility.RandomFloat(0.0, 100.0)
							SoonToBeCorpse.RemoveItem(CurrentForm, SoonToBeCorpse.GetItemCount(CurrentForm))
						EndIf
					EndIf
				EndWhile
			
			Else ; Only do objects
				While i > 0
					i -= 1
					CurrentForm = SoonToBeCorpse.GetNthForm(i)
					If DestructTypes.IsDestructible(CurrentForm)
						StorageUtil.FormListAdd(SoonToBeCorpse, "_MWA_DegradeList", CurrentForm, allowDuplicate = false)
					EndIf
				EndWhile
			EndIf
			
			;Debug.Trace("_MWA_: Init Done on: " + SoonToBeCorpse)
		EndIf
		
		If Menu.RandomNpcFitT
			SoonToBeCorpse.GetLeveledActorBase().SetWeight(Utility.RandomFloat(0.0, 100.0))
			;SoonToBeCorpse.QueueNiNodeUpdate() ; Doesn't work
		EndIf
		
		If SoonToBeCorpse.IsDead()
			;Debug.Trace("_MWA_: Init Degrade starting on: " + SoonToBeCorpse)
			DegradeCorpse()
		EndIf
	EndIf
EndEvent

Function DegradeCorpse()


	Actor Corpse = Self.GetReference() as Actor
	;If StorageUtil.GetIntValue(Corpse, "_MWA_DegradeProc", -1) != 1
		GoToState("Dead")
		;Debug.Messagebox("Degrading Corpse")
		StorageUtil.SetIntValue(Corpse, "_MWA_DegradeProc", 1)
		Corpse.AddItem(_MWA_DegradedToken, 1)
		Int i = StorageUtil.FormListCount(Corpse, "_MWA_DegradeList")
		;Int i = Corpse.GetNumItems()
		Form CurrentForm
		Bool IsItemEquipped = false
		ActorBase CorpseBase = Corpse.GetLeveledActorBase()
		Int Gender = CorpseBase.GetSex()
		Float CorpseWeight = CorpseBase.GetWeight()
		If Gender == 0
			Gender = -1
		EndIf
		
		;Debug.Trace("_MWA_: Degrade: Actor: " + Corpse + ". Item count: " + StorageUtil.FormListCount(Corpse, "_MWA_DegradeList"))
		While i > 0
			i -= 1
			CurrentForm = StorageUtil.FormListGet(Corpse, "_MWA_DegradeList", i)

			If Corpse.IsEquipped(CurrentForm)
				IsItemEquipped = true
			Else
				IsItemEquipped = false
			EndIf
			ObjRef = None
			
			Int CurrentFormCount = Corpse.GetItemCount(CurrentForm)
			Bool IsSingle = true
			If CurrentFormCount > 1
				IsSingle = false
			EndIf
			While CurrentFormCount > 0
				CurrentFormCount -= 1
				AddInventoryEventFilter(CurrentForm)
				Debug.Trace("_MWA_: Dropping " + CurrentForm)
				ObjectReference ObjRefTemp = Corpse.DropObject(CurrentForm) ; Unforunately, DropObject seems unreliable in returning a usable object reference
				;ObjRefTemp.MoveTo(Self.GetReference(), afZOffset = -50.0)
				RemoveInventoryEventFilter(CurrentForm)
				Int j = 300
				If ObjRefTemp == None || StringUtil.Find(ObjRefTemp, "container") != -1
					Debug.Trace("_MWA_: DropObject did not return a usable object reference for " + CurrentForm.GetName() + ". Trying OnItemRemoved")
					While ObjRef == None && j > 0
						j -= 1
						If Utility.IsInMenuMode()
							Utility.WaitMenuMode(0.010)
						Else
							Utility.Wait(0.010)
						EndIf
					EndWhile
					ObjRefTemp = ObjRef
				EndIf
				
				String Name = Util.GetDynamicName(ObjRefTemp, "DegradeCorpse()")
				ObjRefTemp.SetDisplayName(Name) ; Set displayname outside of inventory. Stops objects of different durabilities/fits from stacking
				;Debug.Messagebox("ObjRefTemp: " + ObjRefTemp + ". Name: " + ObjRefTemp.GetBaseObject().GetName() + ". IsEquipped: " + IsItemEquipped + ". IsSingle: " + IsSingle)
				If IsSingle
					Corpse.AddItem(ObjRefTemp, 1, true)
					If IsItemEquipped
						Corpse.EquipItem(CurrentForm)
					EndIf
				Else
					StorageUtil.FormListAdd(Corpse, "_MWA_DroppedObjRefs", ObjRefTemp, allowDuplicate = false)
				EndIf
				
				;Debug.Trace("_MWA_: ObjRefTemp: " + ObjRefTemp + ". Name: " + ObjRefTemp.GetBaseObject().GetName() + ". i: " + i + ". j: " + j)
					
					
				If StorageUtil.GetIntValue(ObjRefTemp, "_MWA_Durability", -1) == -1
					Int Durability = Util.GetDurability(ObjRefTemp, CurrentForm)
					;StorageUtil.SetIntValue(ObjRefTemp, "_MWA_Durability", (Durability - Utility.RandomInt(Math.Floor(Durability/1.5), (Durability - 1)))) ; old durability calc - worse durabilty for corpses...
					StorageUtil.SetIntValue(ObjRefTemp, "_MWA_Durability", Utility.RandomInt(Math.Ceiling(Durability * Menu.CorpseDurabilityMin), Math.Ceiling(Durability * Menu.CorpseDurabilityMax)))
					If !StorageUtil.HasFloatValue(ObjRefTemp, "_MWA_ArmorFit")
						StorageUtil.SetFloatValue(ObjRefTemp, "_MWA_ArmorFit", (Gender * (CorpseWeight + 0.1))) ; + 0.1 because otherwise weight could be 0.0 for both genders
					EndIf
					
					; Curse item?
					If StorageUtil.GetIntValue(ObjRefTemp, "_MWA_IsCursedObject", Missing = -1) < 0 && Menu.CursedChanceCorpse > Utility.RandomFloat(0.0, 100.0)
						StorageUtil.SetIntValue(ObjRefTemp, "_MWA_IsCursedObject", 1)
					Else
						StorageUtil.SetIntValue(ObjRefTemp, "_MWA_IsCursedObject", 0)
					EndIf
					
					Util.DispItemDurability(ObjRefTemp, CurrentForm, 0.0)
					ReferenceAlias AliasSelect = _MWA_NpcLootTrack.GetNthAlias(Init.NpcLootAliasIndex) as ReferenceAlias
					AliasSelect.ForceRefTo(ObjRefTemp)
					Init.NpcLootAliasIndex += 1
					If Init.NpcLootAliasIndex >= Init.NpcLootAliasCount
						Init.NpcLootAliasIndex = 0
					EndIf
				EndIf
			EndWhile
			
			If !IsSingle
				Int j = StorageUtil.FormListCount(Corpse, "_MWA_DroppedObjRefs")
				While j > 0
					j -= 1
					Corpse.AddItem(StorageUtil.FormListGet(Corpse, "_MWA_DroppedObjRefs", j), 1, true)
				EndWhile
				StorageUtil.FormListClear(Corpse, "_MWA_DroppedObjRefs")
				If IsItemEquipped
					Corpse.EquipItemEx(CurrentForm)
				EndIf
			EndIf
		EndWhile
		StorageUtil.FormListClear(Corpse, "_MWA_DroppedObjRefs")
		
		GoToState("")
	;EndIf
	
EndFunction

Bool Property RemoveFood = true Auto Hidden

_MWA_DestructionTypes Property DestructTypes Auto
_MWA_Utility Property Util Auto
_MWA_Init Property Init Auto
_MWA_Mcm Property Menu Auto

Quest Property _MWA_NpcLootTrack Auto 

Armor Property _MWA_DegradedToken Auto
Armor Property _MWA_DummyArmor Auto

Keyword Property VendorItemFood Auto
Keyword Property VendorItemFoodRaw Auto
Keyword Property ActorTypeAnimal Auto
