Scriptname _MWA_PlayerInventoryUtil extends Quest  

_MWA_PlayerInventoryTrackRemoved Property InvRemove Auto
_MWA_Utility Property Util Auto
_MWA_Mcm Property Menu Auto

Quest Property _MWA_PlayerInventoryAliases Auto
Quest Property _MWA_PlayerInventoryCooloff Auto
Quest Property _MWA_NpcLootTrack Auto

Actor Property PlayerRef Auto

Bool[] Block0
Bool[] Block1
Bool[] Block2
Bool[] Block3
Int[] BlockUsage

Bool ReleaseAliasInUse = false
Bool MakePersistentInUse = false
Bool PassToCooloffInUse = false
Bool Property IsShopping = false Auto Hidden

Int BlockSizes = 40
Int CooloffIndex = 0 

; Increasing number of aliases? Increase BlockSizes/Add more Blocks.

Event OnInit()
	Block0 = new Bool[40]
	Block1 = new Bool[40]
	Block2 = new Bool[40]
	Block3 = new Bool[40]
	BlockUsage = new Int[4]
	BlockUsage[0] = 0
	BlockUsage[1] = 0
	BlockUsage[2] = 0
	BlockUsage[3] = 0
	
	FillBoolArrayFalse(Block0)
	FillBoolArrayFalse(Block1)
	FillBoolArrayFalse(Block2)
	FillBoolArrayFalse(Block3)
	
	RegisterForMenu("BarterMenu")
	RegisterForMenu("Crafting Menu")
EndEvent

Function PlayerLoadsGame()
	RegisterForMenu("BarterMenu")
	RegisterForMenu("Crafting Menu")
	CleanInventoryAliases(-1)
	;RegisterForKey(199)
EndFunction
;/
Event OnKeyDown(Int KeyCode)
	RunBlockCleanUp(199)
EndEvent
/;
Event OnMenuOpen(String MenuName)
	IsShopping = true
EndEvent

Event OnMenuClose(String MenuName)
	IsShopping = false
EndEvent

Function FillBoolArrayFalse(Bool[] ArraySelect)
	Int i = ArraySelect.Length
	While i > 0
		i -= 1
		ArraySelect[i] = false
	EndWhile
EndFunction

Function RelayRefRequest(Form akBaseItem)
	While !InvRemove.GetObjectRef(akBaseItem)
	EndWhile
EndFunction

Function GetDurabilityThread(ObjectReference ObjRef, Form akBaseItem)
	Int GetDur = ModEvent.Create("_MWA_Internal_GetDurability")
    If (GetDur)
        ModEvent.PushForm(GetDur, ObjRef as Form)
		ModEvent.PushForm(GetDur, akBaseItem)
		ModEvent.PushBool(GetDur, IsShopping)
        ModEvent.Send(GetDur)
    EndIf
EndFunction

Bool Function MakePersistent(ObjectReference ObjRef, Form akBaseItem)
	If ObjRef != None
		Debug.Trace("_MWA_: MakePersistent: Requested on " + akBaseItem.GetName() + ". ObjRef: " + ObjRef)
		While ReleaseAliasInUse
			;Debug.Trace("_MWA_: MakePersistent: waiting")
			Utility.WaitMenuMode(0.05)
		EndWhile
		If !MakePersistentInUse
			MakePersistentInUse = true
			;Debug.Trace("_MWA_: MakePersistent: Processing on " + akBaseItem.GetName() + ". ObjRef: " + ObjRef)
			If StorageUtil.GetIntValue(ObjRef, "_MWA_InvAliasIndex", missing = -1) == -1 ; double check object was not queued more than once for MakePersistent()
				
				Int i = 0
				While i < BlockUsage.Length
					If BlockUsage[i] < BlockSizes
						;Debug.Messagebox("Selected block " + i + ". Block Usage: " + BlockUsage[i])
						;Debug.Trace("_MWA_: vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv")
						;Debug.Trace("_MWA_: Selected block " + i + ". Block Usage: " + BlockUsage[i])
						;Debug.Trace("_MWA_: " + Block0)
						;Debug.Trace("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^")
						Bool[] BlockSelect
						If i == 0
							BlockSelect = Block0
						ElseIf i == 1
							BlockSelect = Block1
						ElseIf i == 2
							BlockSelect = Block2
						Else
							If BlockUsage[3] < BlockSizes
								BlockSelect = Block3
								
							Else
								Debug.Messagebox("_MWA_: Error: MakePersistent: There are no free aliases remaining")
								MakePersistentInUse = false
								Return true
							EndIf
						EndIf
						
						; add to alias
						Int Index = BlockSelect.Find(false)
						If Index > -1
							Int AliasIndex = i * BlockSizes + Index
							ReferenceAlias AliasSelect = _MWA_PlayerInventoryAliases.GetNthAlias(AliasIndex) as ReferenceAlias
							If AliasSelect.GetReference() == None
								AliasSelect.ForceRefTo(ObjRef)
								StorageUtil.SetIntValue(ObjRef, "_MWA_InvAliasIndex", AliasIndex)
								BlockSelect[Index] = true
								;Debug.Trace("_MWA_: MakePersistent: " + akBaseItem.GetName() + ". Ref: ObjRef: " + ObjRef + ". _MWA_InvAliasIndex: " + StorageUtil.GetIntValue(ObjRef, "_MWA_InvAliasIndex", -1))
								;GetDurabilityThread(ObjRef, akBaseItem) ; Thread off GetDurability to free up MakePersistent() earlier

								;If StorageUtil.GetIntValue(akBaseItem, "_MWA_MakePersistMultipleProc") > 1 ; If you have more than one of this item get their durability now to stop them stacking in the inventory
									;Debug.Messagebox("Doing more than one")
								;	Util.GetDurability(ObjRef, akBaseItem, ForceUpdate = false, IsWorn = !IsShopping)
								;Else
									;Debug.Messagebox("Doing one")
									GetDurabilityThread(ObjRef, akBaseItem) ; Thread off GetDurability to free up MakePersistent() earlier
								;EndIf
								
								; Setting the name stops the objects stacking in the inventory but there appears to be 2 display names. If name is set in the world that name will be used when object is in the world and not anything set when the object was in inventory...
								;If Menu.AddDurabilityDes ; Set the objects name to whatever while it's out of inventory otherwise 2x BaseItems will stack in inventory
									;Util.DispItemDurability (ObjRef, akBaseItem, 0.0);		ObjRef.SetDisplayName(akBaseItem.GetName())
								;EndIf
							Else
								Debug.Messagebox("MakePersistent: something already in alias")
							EndIf
						Else
						
							Debug.Trace("_MWA_: ERROR: BlockUsage and BlockSelect[] actual discrepency. Running Block cleanup")
							Debug.Messagebox("_MWA_: ERROR: BlockUsage and BlockSelect[] actual discrepency. Running Block cleanup, this may take a moment. Your item has been dropped. Once you get 'Block cleanup success message' you can try picking it up again")
							ObjRef.MoveTo(PlayerRef, 70.0 * Math.Sin(PlayerRef.GetAngleZ()), 70.0 * Math.Cos(PlayerRef.GetAngleZ()), PlayerRef.GetHeight() + 20.0)
							
							CleanInventoryAliases(i)
							;/
							While true
								Debug.Messagebox("_MWA_: MakePersistent: no free space. Index: " + Index + ". BlockUsage[i]: " + BlockUsage[i] + ". i: " + i + ". BlockSelect: " + BlockSelect)
								Debug.Trace("_MWA_: MakePersistent: no free space. Index: " + Index + ". BlockUsage[i]: " + BlockUsage[i] + ". i: " + i + ". BlockSelect: " + BlockSelect)
							EndWhile
							/;
						EndIf
						Debug.Trace("_MWA_: New: MakePersistent: Returning true on " + ObjRef)
						BlockUsage[i] = BlockUsage[i] + 1
						MakePersistentInUse = false
						Return true
					
					ElseIf i == BlockUsage.Length - 1 && BlockUsage[i] >= BlockSizes
						Debug.Trace("_MWA_: Error: MakePersistent: There are no free aliases remaining")
						Debug.Notification("I simply can not carry any more")
						ObjRef.MoveTo(PlayerRef, 70.0 * Math.Sin(PlayerRef.GetAngleZ()), 70.0 * Math.Cos(PlayerRef.GetAngleZ()), PlayerRef.GetHeight() + 20.0)
						MakePersistentInUse = false
						Return true
					EndIf
					i += 1
				EndWhile
				
				MakePersistentInUse = false
				
			Else
				;Debug.Trace("_MWA_: Warning: " + akBaseItem.GetName() + " already has an alias index. ObjRef: " + ObjRef)
				MakePersistentInUse = false
				Return true
			EndIf
			
		Else ; wait
			Utility.Wait(0.01)
			Debug.Trace("_MWA_: New: MakePersistent is too busy to process " + ObjRef)
			Return false
		EndIf
	
	Else
		Debug.Trace("_MWA_: New: Error: MakePersistent: ObjRef = None")
		Return true
	EndIf
EndFunction

Function PrintBlockUsage()
	Debug.Trace("_MWA_: Block: Block dump BEGIN vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv")
	Int i = 0
	Debug.Trace("_MWA_: Block: BlockUsage0: " + BlockUsage[0])
	While i < Block0.Length
		Debug.Trace("_MWA_: Block: Block0 " + i + ") " + Block0[i])
		i += 1
	EndWhile
	
	i = 0
	Debug.Trace("_MWA_: Block: BlockUsage1: " + BlockUsage[1])
	While i < Block1.Length
		Debug.Trace("_MWA_: Block: Block1 " + i + ") " + Block1[i])
		i += 1
	EndWhile
	
	i = 0
	Debug.Trace("_MWA_: Block: BlockUsage2: " + BlockUsage[2])
	While i < Block2.Length
		Debug.Trace("_MWA_: Block: Block2 " + i + ") " + Block2[i])
		i += 1
	EndWhile
	
	i = 0
	Debug.Trace("_MWA_: Block: BlockUsage3: " + BlockUsage[3])
	While i < Block3.Length
		Debug.Trace("_MWA_: Block: Block3 " + i + ") " + Block3[i])
		i += 1
	EndWhile
	
	Debug.Trace("_MWA_: Block: Alias Dump begin ----------------------------------------------------------------------------")
	i = 0
	ReferenceAlias AliasSelect
	While i < _MWA_PlayerInventoryAliases.GetNumAliases()
		AliasSelect = _MWA_PlayerInventoryAliases.GetNthAlias(i) as ReferenceAlias
		Debug.Trace("_MWA_: Block: Alias " + i + ") " + AliasSelect.GetReference())
		i += 1
	EndWhile
	Debug.Trace("_MWA_: Block: Alias Dump END ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^")
EndFunction

Function ReleaseAlias(Int Index, ObjectReference ObjRef)
	If ObjRef
		If !ObjRef.IsDisabled() ; ArmorDestruction disables objref before removing from inventory. Don't bother cooling off these
			While !PassToCooloff(ObjRef)
			EndWhile
		EndIf
		While MakePersistentInUse
			Debug.Trace("_MWA_: ReleaseAlias: waiting")
			Utility.WaitMenuMode(0.05)
		EndWhile
		ReferenceAlias AliasSelect = _MWA_PlayerInventoryAliases.GetNthAlias(Index) as ReferenceAlias
		If AliasSelect.GetReference() == ObjRef
			ReleaseAliasInUse = true
			StorageUtil.UnSetIntValue(ObjRef, "_MWA_InvAliasIndex")
			AliasSelect.Clear()
			Bool[] BlockSelect = GetOwningBlock(Index, ModifyBlockUsage = -1)
			BlockSelect[GetBlockIndex(Index)] = false
			;PrintBlockUsage()
			ReleaseAliasInUse = false
		Else
			Debug.Trace("_MWA_: New: Error: ReleaseAlias: AliasSelect (" + AliasSelect.GetReference() + ") does not match ObjRef: " + ObjRef)
		EndIf
	
	Else
		Debug.Trace("_MWA_: Warning: ReleaseAlias: Received a None ObjRef")
	EndIf
EndFunction

Bool[] Function GetOwningBlock(Int Index, Int ModifyBlockUsage = 0) ; Returns the block owning the index. ModifyBlockUsage = What to change BlockUsage by (usuaully -1)
	Int BlockSelect = Math.Floor(Index / BlockSizes)
	If BlockSelect == 0
		BlockUsage[0] = BlockUsage[0] - ModifyBlockUsage
		Return Block0
	ElseIf BlockSelect == 1
		BlockUsage[1] = BlockUsage[1] - ModifyBlockUsage
		Return Block1
	ElseIf BlockSelect == 2
		BlockUsage[2] = BlockUsage[2] - ModifyBlockUsage
		Return Block2
	ElseIf BlockSelect == 3
		BlockUsage[3] = BlockUsage[3] - ModifyBlockUsage
		Return Block3
	Else
		Debug.Trace("_MWA_: New: Error: GetOwningBlock: Index out of range" + Index)
	EndIf
EndFunction

Int Function GetBlockIndex(Int Index)
	Return (Index - (Math.Floor(Index / BlockSizes) * BlockSizes))
EndFunction

Bool Function PassToCooloff(ObjectReference ObjRef)
	If !PassToCooloffInUse ; Is serialization necessary ?
		PassToCooloffInUse = true
		
		If !IsShopping
			ReferenceAlias AliasSelect
			Int i = ObjRef.GetNumReferenceAliases()
			If i > 0 
				While i > 0
					i -= 1
					AliasSelect = ObjRef.GetNthReferenceAlias(i)
					If AliasSelect.GetOwningQuest() == _MWA_PlayerInventoryCooloff
						Debug.Trace("_MWA_: " + ObjRef + " is already cooling off")
						PassToCooloffInUse = false
						Return true
					EndIf
				EndWhile
			EndIf
			AliasSelect = _MWA_PlayerInventoryCooloff.GetNthAlias(CooloffIndex) as ReferenceAlias
			AliasSelect.ForceRefTo(ObjRef)
			CooloffIndex += 1
			If CooloffIndex > _MWA_PlayerInventoryCooloff.GetNumAliases() - 1
				CooloffIndex = 0
			EndIf
		EndIf
		PassToCooloffInUse = false
		Return true
	Else
		Debug.Trace("_MWA_: New: PassToCooloff: Is busy waiting - " + ObjRef)
		Utility.WaitMenuMode(0.02)
		Return false
	EndIf
EndFunction

Function CleanInventoryAliases(Int i)
	;Debug.Notification("_MWA_: Alias cleanup in progress")
	ObjectReference ObjRef
	ReferenceAlias AliasSelect
	Form akBaseObject
	Int k = _MWA_PlayerInventoryAliases.GetNumAliases()
	While ReleaseAliasInUse
		Utility.WaitMenuMode(2.0)
	EndWhile
	ReleaseAliasInUse = true
	Debug.Trace("_MWA_: BEGIN cleaning PlayerInventoryAliases")
	While k > 0
		k -= 1
		;Debug.Trace("_MWA_: Checking alias " + k ") ")
		AliasSelect = _MWA_PlayerInventoryAliases.GetNthAlias(k) as ReferenceAlias
		ObjRef = AliasSelect.GetReference()
		If ObjRef != None
			;Debug.Trace("_MWA_: CleanInventoryAliases ObjRef: " + ObjRef)
			;Debug.Trace("_MWA_: CleanInventoryAliases Base: " + akBaseObject.GetName())
			;akBaseObject = ObjRef.GetBaseObject()
			;Debug.Trace("_MWA_: CleanInventoryAliases: " + akBaseObject.GetName() + " - " + ObjRef + ". Disabled: " + ObjRef.IsDisabled() + ". Deleted: " + ObjRef.IsDeleted() + ". ItemCount: " + PlayerRef.GetItemCount(ObjRef))
			;If ObjRef.IsDeleted() || ObjRef.IsDisabled()
				If StringUtil.Find(ObjRef, "in container") > -1 || PlayerRef.GetItemCount(ObjRef) == 0; GetItemCount appears to work for ObjRefs. 2x Steel greatswords returns a count of 1 for each greatsword ref. Amazing!
					;Debug.Trace("_MWA_: Player definitely doesn't have one of these. Deleting")
					Bool[] BlockSelect = GetOwningBlock(k, ModifyBlockUsage = 0)
					StorageUtil.UnSetIntValue(ObjRef, "_MWA_InvAliasIndex")
					BlockSelect[GetBlockIndex(k)] = false
					AliasSelect.Clear()
				EndIf
			;EndIf
		EndIf
	EndWhile
	RunBlockCleanUp(i)
	;Debug.Messagebox("_MWA_: BlockUsage[0]: " + BlockUsage[0])
	;Debug.Trace("_MWA_: BlockUsage[0]: " + BlockUsage[0] + ". Block0[]: " + Block0)
	ReleaseAliasInUse = false
	Debug.Notification("_MWA_: Alias cleanup complete")
	Debug.Trace("_MWA_: END cleaning PlayerInventoryAliases")
EndFunction

Function RunBlockCleanUp(Int i)
	BlockUsage[0] = 0
	BlockUsage[1] = 0
	BlockUsage[2] = 0
	BlockUsage[3] = 0
	
	FillBoolArrayFalse(Block0)
	FillBoolArrayFalse(Block1)
	FillBoolArrayFalse(Block2)
	FillBoolArrayFalse(Block3)

	Int k = _MWA_PlayerInventoryAliases.GetNumAliases()
	ReferenceAlias AliasSelect
	Bool[] BlockSelect
	While k > 0
		k -= 1
		AliasSelect = _MWA_PlayerInventoryAliases.GetNthAlias(k) as ReferenceAlias
		If AliasSelect.GetReference() != None
			BlockSelect = GetOwningBlock(k, ModifyBlockUsage = -1)
			BlockSelect[GetBlockIndex(k)] = true
		EndIf
	EndWhile
	If i != -1
		BlockUsage[i] = BlockUsage[i] - 1 ; Counteracts 'BlockUsage[i] = BlockUsage[i] + 1' that's run after RunBlockCleanUp()
	EndIf
	Debug.Trace("_MWA_: Block cleanup complete. Ok to go")
	;Debug.Messagebox("_MWA_: Block cleanup complete. Ok to go")
EndFunction

Function ClearImmuneObjectsFromInvList(Form akBaseObject) ; Used to remove objects added to the immune list from inventory aliases. Also renames them back
	Debug.Notification("_MWA_: Start Removing " + akBaseObject.GetName() + " objects from inventory aliases")
	Debug.Trace("_MWA_: Start Removing " + akBaseObject.GetName() + " objects from inventory aliases")
	ObjectReference ObjRef
	Form ObjRefBase
	Int i = _MWA_PlayerInventoryAliases.GetNumAliases()
	While i > 0
		i -= 1
		ObjRef = (_MWA_PlayerInventoryAliases.GetNthAlias(i) as ReferenceAlias).GetReference()
		If ObjRef
			ObjRefBase = ObjRef.GetBaseObject()
			If ObjRefBase == akBaseObject
				Menu.AddImmunityUpdate(i, true)
				ObjRef.SetDisplayName(Util.GetDynamicName(ObjRef, "ClearImmuneObjectsFromInvList"))
				ObjRef.MoveTo(PlayerRef)
				ObjRef.SetDisplayName(Util.GetDynamicName(ObjRef, "ClearImmuneObjectsFromInvList"))
				PlayerRef.AddItem(ObjRef)
				StorageUtil.ClearAllObjPrefix(ObjRef, "_MWA_")
				ReleaseAlias(i, ObjRef)
				RemoveFromCooloffAliases(ObjRef)
				RemoveFromNpcLootAliases(ObjRef)
			EndIf
		EndIf
		Menu.AddImmunityUpdate(i, false)
	EndWhile
	Menu.AddImmunityUpdate(-1, false)
	Debug.Notification("_MWA_: Finished Removing " + akBaseObject.GetName() + " objects from inventory aliases")
EndFunction

Function RemoveFromNpcLootAliases(ObjectReference ObjRef)
	Int i = _MWA_NpcLootTrack.GetNumAliases()
	ReferenceAlias AliasSelect
	While i > 0
		i -= 1
		AliasSelect = (_MWA_NpcLootTrack.GetNthAlias(i) as ReferenceAlias)
		If AliasSelect.GetReference() == ObjRef
			AliasSelect.Clear()
		EndIf
	EndWhile
EndFunction

Function RemoveFromCooloffAliases(ObjectReference ObjRef)
	Int i = _MWA_PlayerInventoryCooloff.GetNumAliases()
	ReferenceAlias AliasSelect
	While i > 0
		i -= 1
		AliasSelect = (_MWA_PlayerInventoryCooloff.GetNthAlias(i) as ReferenceAlias)
		If AliasSelect.GetReference() == ObjRef
			AliasSelect.Clear()
		EndIf
	EndWhile
EndFunction
