Scriptname _MWA_RepairContainer extends ObjectReference  

Event OnInit()

EndEvent

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	If !akBaseItem.HasKeyword(_MWA_Broken)
	
		; Not valid for repair
		Debug.Notification(akBaseItem.GetName() + " is not repairable")
		GiveBack(akBaseItem, akItemReference, aiItemCount)
		
	Else
		Int ItemRepairCost
		If (Init.RepairActor.IsInFaction(_MWA_TailorFaction) && _MWA_BrokenSmithList.HasForm(akBaseItem))
			Debug.Notification("Only a smith can repair this item")
			GiveBack(akBaseItem, akItemReference, aiItemCount)
		ElseIf Init.RepairActor.IsInFaction(JobBlacksmithFaction) && _MWA_BrokenTailorList.HasForm(akBaseItem)
			Debug.Notification("Only a tailor can repair this item")
			GiveBack(akBaseItem, akItemReference, aiItemCount)
		Else
			; Repairable
			ItemRepairCost = Menu.RepairCostMin + Math.Ceiling((StorageUtil.GetIntValue(akItemReference, "_MWA_OriginalValue") * Menu.RepairCostBase) + (StorageUtil.GetIntValue(akItemReference, "_MWA_OriginalValueEnchant") * Menu.RepairCostEnchant))
			_MWA_FixCost.SetValueInt(_MWA_FixCost.GetValueInt() + ItemRepairCost)
			StorageUtil.FormListAdd(none, "_MWA_ForRepairList",  akItemReference, allowDuplicate = false)
			Debug.Notification("Repair cost: " + ItemRepairCost + ". Total cost: " + _MWA_FixCost.GetValueInt() + ". Your gold: " + PlayerRef.GetItemCount(Gold001) + ". Remaining gold: " + (PlayerRef.GetItemCount(Gold001) - _MWA_FixCost.GetValueInt()))
		EndIf
	EndIf
EndEvent


Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	If akBaseItem.HasKeyword(_MWA_Broken)
		Int ItemRepairCost = Menu.RepairCostMin + Math.Ceiling((StorageUtil.GetIntValue(akItemReference, "_MWA_OriginalValue") * Menu.RepairCostBase) + (StorageUtil.GetIntValue(akItemReference, "_MWA_OriginalValueEnchant") * Menu.RepairCostEnchant))
		_MWA_FixCost.SetValueInt(_MWA_FixCost.GetValueInt() - ItemRepairCost)
		StorageUtil.FormListRemove(none, "_MWA_ForRepairList", akItemReference, allInstances = true)
		Debug.Notification("Repair cost reduced by: " + ItemRepairCost + ". Total cost: " + _MWA_FixCost.GetValueInt() + ". Your gold: " + PlayerRef.GetItemCount(Gold001) + ". Remaining gold: " + (PlayerRef.GetItemCount(Gold001) - _MWA_FixCost.GetValueInt()))
	EndIf
EndEvent

State IgnoreRemoval
	Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	
	EndEvent
EndState

Event OnActivate(ObjectReference akActionRef)
	If akActionRef == PlayerRef
		RegisterForMenu("ContainerMenu")
	EndIf
EndEvent

Event OnMenuClose(String MenuName)
	If Self.GetNumItems() > 0
		If PlayerRef.GetItemCount(Gold001) >= _MWA_FixCost.GetValueInt()
		
			Int Button = _MWA_RepairConfirm.Show(afArg1 = _MWA_FixCost.GetValue())
			If Button == 0 ; Proceed
				PlayerRef.RemoveItem(akItemToRemove = Gold001, aiCount = _MWA_FixCost.GetValueInt(), abSilent = false, akOtherContainer = Init.RepairActor)
				DoRepairs()
			Else ; cancel
				GoToState("IgnoreRemoval")
				Self.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
				GoToState("")
			Endif
		Else
			Debug.Notification("You can't afford that")
			GoToState("IgnoreRemoval")
			Self.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
			GoToState("")
		EndIf
		
		; Reset
		StorageUtil.FormListClear(none, "_MWA_ForRepairList")
		GoToState("IgnoreRemoval")
		Self.RemoveAllItems()
		GoToState("")
		_MWA_FixCost.SetValueInt(0)
	EndIf
	UnregisterForMenu("ContainerMenu")
EndEvent

Function DoRepairs()
	Int i = StorageUtil.FormListCount(none, "_MWA_ForRepairList")
	ObjectReference DummyRef
	ObjectReference NewRef
	Form OrigItem
	Int OrigDurability
	Enchantment E
	While i > 0
		i -= 1
		
		DummyRef = StorageUtil.FormListGet(none, "_MWA_ForRepairList", i) as ObjectReference
		;Debug.Trace("_MWA_: " + i + " - " + DummyRef)
		OrigItem = StorageUtil.GetFormValue(DummyRef, "_MWA_OriginalArmor")
		OrigDurability = StorageUtil.GetIntValue(DummyRef, "_MWA_DurabilityMax")
		E = StorageUtil.GetFormValue(DummyRef, "_MWA_OriginalEnchantment") as Enchantment
		Float OrigCharge = StorageUtil.GetFloatValue(DummyRef, "_MWA_OriginalCharge")
		Float OrigChargeMax = StorageUtil.GetFloatValue(DummyRef, "_MWA_OriginalChargeMax")
		;Debug.MessageBox("OrigCharge: " + OrigCharge + ". MaxCharge: " + OrigChargeMax)
		NewRef = PlayerRef.PlaceAtMe(OrigItem, 1)
		StorageUtil.SetIntValue(NewRef, "_MWA_Durability", OrigDurability)
		StorageUtil.SetIntValue(NewRef, "_MWA_DurabilityMax", OrigDurability)
		StorageUtil.SetStringValue(NewRef, "_MWA_ObjDisplayName", StorageUtil.GetStringValue(DummyRef, "_MWA_ObjDisplayName"))
		Util.FitToPlayer(NewRef, OrigItem)
		Util.DispItemDurability(NewRef, OrigItem, 0.0)
		Int ItemCount = PlayerRef.GetItemCount(OrigItem)
		
		Enchantment BaseEnchant
		If OrigItem as Armor
			BaseEnchant = (OrigItem as Armor).GetEnchantment()
		Else
			BaseEnchant = (OrigItem as Weapon).GetEnchantment()
		EndIf
		If E != None && BaseEnchant == none ; Is a player-made enchantment
			Int SlotMask = 0 
			Int HandSlot = 0
			If OrigItem as Armor
				SlotMask = (OrigItem as Armor).GetSlotMask()
			Else
				HandSlot = 1
			EndIf
			Form CurrentEquip
			If OrigItem as Armor
				CurrentEquip = PlayerRef.GetWornForm(SlotMask)
			Else
				CurrentEquip = PlayerRef.GetEquippedWeapon()
			EndIf
			If (ItemCount > 0)
				PlayerRef.RemoveItem(OrigItem, ItemCount, true, _MWA_TempEquipBarrelRef)
			EndIf
			PlayerRef.AddItem(NewRef)
			;_MWA_EquipsToIgnore.AddForm(NewRef)
			PlayerRef.EquipItemEx(OrigItem, 0, false, false)
			Utility.Wait(0.2)
			Int j = 0
			If OrigItem as Armor
				While PlayerRef.GetWornForm(SlotMask) != OrigItem && j < 50 ; Enchantment is being set before armor is equipped...? wtf
					Utility.Wait(0.1)
					j += 1
				EndWhile
			Else
				While PlayerRef.GetEquippedWeapon() != OrigItem && j < 50 ; Enchantment is being set before armor is equipped...? wtf
					Utility.Wait(0.1)
					j += 1
				EndWhile
			EndIf
			;Debug.MessageBox(OrigChargeMax)
			If OrigItem as Armor
				WornObject.SetEnchantment(PlayerRef, HandSlot, SlotMask, E, OrigChargeMax) ; Set enchantment on weapons creates an enchantment ok but the charge never depletes.... er Hurray!
			Else
				WornObject.SetEnchantment(PlayerRef, HandSlot, SlotMask, E, OrigChargeMax)
				
				;Util.CreateWeaponEnchantment(E, HandSlot, OrigChargeMax)
				Debug.Trace("_MWA_: Dropping " + OrigItem)
				NewRef = PlayerRef.DropObject(OrigItem, 1)
				NewRef.SetItemCharge(OrigCharge)
				PlayerRef.AddItem(NewRef, 1, true)
				
			EndIf
			;If CurrentEquip == None
			;	PlayerRef.UnEquipItem(PlayerRef.GetWornForm(SlotMask), false, true)
			;EndIf
			_MWA_TempEquipBarrelRef.RemoveAllItems(akTransferTo = PlayerRef, abKeepOwnership = true, abRemoveQuestItems = false)
			If CurrentEquip != none
				PlayerRef.EquipItemEx(CurrentEquip, 0, preventUnequip = false, equipSound = false)
			EndIf
		Else
			PlayerRef.AddItem(NewRef)
		Endif
		Util.UnPersist(DummyRef, _MWA_BrokenTrack, "_MWA_BrokenTrackAliasList", Init.BrokenTrackAliasCount)
		
	EndWhile
EndFunction

Function GiveBack(Form akBaseItem, ObjectReference akItemReference, Int aiItemCount)
	GoToState("IgnoreRemoval")
	If akItemReference
		Self.RemoveItem(akItemReference, aiItemCount, true, PlayerRef)
	Else
		Self.RemoveItem(akBaseItem, aiItemCount, true, PlayerRef)
	EndIf
	GoToState("")
EndFunction

Faction Property _MWA_TailorFaction Auto
Faction Property JobBlacksmithFaction Auto

Formlist Property _MWA_BrokenTailorList Auto
Formlist Property _MWA_BrokenSmithList Auto
Formlist Property _MWA_EquipsToIgnore Auto

ObjectReference Property _MWA_TempEquipBarrelRef Auto

Keyword Property _MWA_Broken Auto

Actor Property PlayerRef Auto

GlobalVariable Property _MWA_FixCost Auto

MiscObject Property Gold001 Auto

Message Property _MWA_RepairConfirm Auto

Quest Property _MWA_BrokenTrack Auto

_MWA_Init Property Init Auto
_MWA_Utility Property Util Auto
_MWA_Mcm Property Menu Auto
